==> toycars-0.3.0.tar.gz: toycars-0.3.0/src/KnockOutGame.cpp <==
/*
 *  KnockOutGame.cpp
 *  ToyCars
 *
 *  Created by Ruben Zilibowitz on 17/12/05.
 *  Copyright 2005 Ruben Zilibowitz. All rights reserved.
 *
 */

#include "KnockOutGame.h"
#include "GameConstants.h"
#include "ScException.h"
#include "glf.h"
#include "TcPreferences.h"

extern TcPreferences *gPreferences;

KnockOutGame::KnockOutGame(ScTilemap* inTilemap, Player** inPlayers, int inNumPlayers, Tuple inStartLineA, Tuple inStartLineB, const list<Tuple> *inPathPolygon)
    :   CarGame(inTilemap, inPlayers, inNumPlayers, inStartLineA, inStartLineB, inPathPolygon),
        scores(new int[inNumPlayers])
{
/*
	PlayerEventKey playerEvent;
	playerEvent.type = kPlayerKeyDown;
	playerEvent.key = kThrottleMore;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++)
	{
        allPlayersList.push_back(*i);
		(*i)->doEvent(&playerEvent);
	}
*/
    for (int i = 0; i < numPlayers; i++)
        scores[i] = 0;
}

KnockOutGame::~KnockOutGame()
{
    delete scores;
}

void KnockOutGame::addView(ScPortRect& portRect, int playerFollow)
{
    CarGame::addView(portRect, playerFollow);
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}

const double kWinnerScrollingBias = 0.8;	// must be between 0 and 1
const double kOneMinusScrollBias = 1 - kWinnerScrollingBias;

void KnockOutGame::step(double dt)
{
    ScView* view = views.front().view;
    
    // scroll the view to the car location
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle += (*i)->car.getLocation_wc() * kScale;
    }
    middle /= playersInGame.size();
    
	double bestDist = -100;
	Player *winning = NULL;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		if ((*i)->getPathDistance() > bestDist) {
			winning = *i;
			bestDist = (*i)->getPathDistance();
		}
	}
	middle = kOneMinusScrollBias*middle + kWinnerScrollingBias*winning->car.getLocation_wc()*kScale;
    
    middle.x -= (view->getViewRect().left + view->getViewRect().right)*0.5;
    middle.y -= (view->getViewRect().top + view->getViewRect().bottom)*0.5;
	middle *= 0.05;
	/*
    double d = middle.Length();
    if (d > 5000*dt)
        middle *= 5000*dt / d;
	*/
    view->scrollBy(floor(middle.x), floor(middle.y));
    
    CarGame::step(dt);
}

void KnockOutGame::drawViews()
{
    ScView* view = views.front().view;
    
    // bound the view to the animation bounds
    
    if (view->getViewRect().left < gameRect.left)
        view->scrollBy(gameRect.left - view->getViewRect().left, 0);
    
    if (view->getViewRect().bottom < gameRect.bottom)
        view->scrollBy(0, gameRect.bottom - view->getViewRect().bottom);
    
    if (view->getViewRect().right > gameRect.right)
        view->scrollBy(gameRect.right - view->getViewRect().right, 0);
    
    if (view->getViewRect().top > gameRect.top)
        view->scrollBy(0, gameRect.top - view->getViewRect().top);
    
    // draw scene
    view->draw();
    
    // draw scores
    glLoadIdentity();
    glOrtho(0, view->getViewRect().right - view->getViewRect().left, 0, view->getViewRect().top - view->getViewRect().bottom, -1, 1);
    static char str15[16];
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glColor4f(1.0f,1.0f,1.0f,0.5f);
    glPushMatrix();
    glTranslatef(24, gPreferences->getScreenHeight()-96, 0);
    glScalef(10, 10, 1);
    for (int i = 0; i < numPlayers; i++) {
        sprintf(str15, "%d", scores[i]);
        glfDrawSolidString(str15);
        glTranslatef(0, -4.8, 0);
    }
    glPopMatrix();
    
    // check for knock outs
	
    double curLaps, bestLaps = -2;
    Player *winning = NULL;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		//(*i)->computePathLocation();
		(*i)->updatePathLocation();
		curLaps = (*i)->getPathDistance();
        if (curLaps > bestLaps) {
            winning = *i;
            bestLaps = curLaps;
        }
    }
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        if (    (*i)->car.getBounds().left < view->getViewRect().left ||
                (*i)->car.getBounds().right > view->getViewRect().right ||
                (*i)->car.getBounds().top > view->getViewRect().top ||
                (*i)->car.getBounds().bottom < view->getViewRect().bottom)
        {
            if ((*i) != winning) {
                Player *removal = *i;
                i--;
                playersInGame.remove(removal);
				allPlayersList.remove(removal);
				allPlayersList.push_front(removal);
				//printf("removed player had path distance of %lf\n", removal->getPathDistance());
                removal->car.setVisible(false);
                if (playersInGame.size() == 1) {
                    scores[winning->getPlayerNumber() - 1]++;
					allPlayersList.remove(winning);
					allPlayersList.push_front(winning);
                    restartFromWinner(winning);
					//printf("winning player had path distance of %lf\n", winning->getPathDistance());
                    break;
                }
            }
        }
    }
}

void KnockOutGame::restartFromWinner(Player *winner)
{
    Tuple placement, loc = winner->getPathLocation(), hOffset, vOffset = winner->getPathVector();
	double theta;
	
	vOffset.Normalise();
	hOffset = vOffset.perp();
	theta = atan2(vOffset.y, vOffset.x);
	//loc = loc * (tilemap.getTileWidth() / kScale);
	short count = 0;
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
		if (count % 2 == 1) {
			placement = loc - hOffset * (0.55 * (*i)->car.getHeight() / kScale);
			loc -= vOffset * ((*i)->car.getWidth() / kScale);
		}
		else {
			placement = loc + hOffset * (0.55 * (*i)->car.getHeight() / kScale);
		}
        (*i)->car.setLocation(placement);
		(*i)->car.setOrientation(theta);
        (*i)->car.setVisible(true);
		(*i)->car.kill();
		count++;
    }
    
    playersInGame.clear();
	winner->updatePathLocation();
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
        playersInGame.push_back(*i);
        (*i)->reset(startLineA, startLineB, winner);
    }
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}

==> toycars-0.3.2.tar.gz: toycars-0.3.2/src/KnockOutGame.cpp <==
/*
 *  KnockOutGame.cpp
 *  ToyCars
 *
 *  Created by Ruben Zilibowitz on 17/12/05.
 *  Copyright 2005 Ruben Zilibowitz. All rights reserved.
 *
 */

#include "KnockOutGame.h"
#include "GameConstants.h"
#include "ScException.h"
#include "glf.h"
#include "TcPreferences.h"

extern TcPreferences *gPreferences;

KnockOutGame::KnockOutGame(ScTilemap* inTilemap, Player** inPlayers, int inNumPlayers, Tuple inStartLineA, Tuple inStartLineB, const list<Tuple> *inPathPolygon)
    :   CarGame(inTilemap, inPlayers, inNumPlayers, inStartLineA, inStartLineB, inPathPolygon),
        scores(new int[inNumPlayers])
{
/*
	PlayerEventKey playerEvent;
	playerEvent.type = kPlayerKeyDown;
	playerEvent.key = kThrottleMore;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++)
	{
        allPlayersList.push_back(*i);
		(*i)->doEvent(&playerEvent);
	}
*/
    for (int i = 0; i < numPlayers; i++)
        scores[i] = 0;
}

KnockOutGame::~KnockOutGame()
{
    delete scores;
}

void KnockOutGame::addView(ScPortRect& portRect, int playerFollow)
{
    CarGame::addView(portRect, playerFollow);
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}

const double kWinnerScrollingBias = 0.8;	// must be between 0 and 1
const double kOneMinusScrollBias = 1 - kWinnerScrollingBias;

void KnockOutGame::step(double dt)
{
    ScView* view = views.front().view;
    
    // scroll the view to the car location
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle += (*i)->car.getLocation_wc() * kScale;
    }
    middle /= playersInGame.size();
    
	double bestDist = -100;
	Player *winning = NULL;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		if ((*i)->getPathDistance() > bestDist) {
			winning = *i;
			bestDist = (*i)->getPathDistance();
		}
	}
	middle = kOneMinusScrollBias*middle + kWinnerScrollingBias*winning->car.getLocation_wc()*kScale;
    
    middle.x -= (view->getViewRect().left + view->getViewRect().right)*0.5;
    middle.y -= (view->getViewRect().top + view->getViewRect().bottom)*0.5;
	middle *= 0.05;
	/*
    double d = middle.Length();
    if (d > 5000*dt)
        middle *= 5000*dt / d;
	*/
    view->scrollBy(floor(middle.x), floor(middle.y));
    
    CarGame::step(dt);
}

void KnockOutGame::drawViews()
{
    ScView* view = views.front().view;
    
    // bound the view to the animation bounds
    
    if (view->getViewRect().left < gameRect.left)
        view->scrollBy(gameRect.left - view->getViewRect().left, 0);
    
    if (view->getViewRect().bottom < gameRect.bottom)
        view->scrollBy(0, gameRect.bottom - view->getViewRect().bottom);
    
    if (view->getViewRect().right > gameRect.right)
        view->scrollBy(gameRect.right - view->getViewRect().right, 0);
    
    if (view->getViewRect().top > gameRect.top)
        view->scrollBy(0, gameRect.top - view->getViewRect().top);
    
    // draw scene
    view->draw();
    
    // draw scores
    glLoadIdentity();
    glOrtho(0, view->getViewRect().right - view->getViewRect().left, 0, view->getViewRect().top - view->getViewRect().bottom, -1, 1);
    static char str15[16];
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glColor4f(1.0f,1.0f,1.0f,0.5f);
    glPushMatrix();
    glTranslatef(24, gPreferences->getScreenHeight()-96, 0);
    glScalef(10, 10, 1);
    for (int i = 0; i < numPlayers; i++) {
        sprintf(str15, "%d", scores[i]);
        glfDrawSolidString(str15);
        glTranslatef(0, -4.8, 0);
    }
    glPopMatrix();
    
    // check for knock outs
	
    double curLaps, bestLaps = -2;
    Player *winning = NULL;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		//(*i)->computePathLocation();
		(*i)->updatePathLocation();
		curLaps = (*i)->getPathDistance();
        if (curLaps > bestLaps) {
            winning = *i;
            bestLaps = curLaps;
        }
    }
    
	const ScRect* carBounds;
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
		carBounds = &(*i)->car.getBounds();
        if (    carBounds->right < view->getViewRect().left ||
                carBounds->left > view->getViewRect().right ||
                carBounds->bottom > view->getViewRect().top ||
                carBounds->top < view->getViewRect().bottom)
        {
            if ((*i) != winning) {
                Player *removal = *i;
                i--;
                playersInGame.remove(removal);
				allPlayersList.remove(removal);
				allPlayersList.push_front(removal);
				//printf("removed player had path distance of %lf\n", removal->getPathDistance());
                removal->car.setVisible(false);
                if (playersInGame.size() == 1) {
                    scores[winning->getPlayerNumber() - 1]++;
					allPlayersList.remove(winning);
					allPlayersList.push_front(winning);
                    restartFromWinner(winning);
					//printf("winning player had path distance of %lf\n", winning->getPathDistance());
                    break;
                }
            }
        }
    }
}

void KnockOutGame::restartFromWinner(Player *winner)
{
    Tuple placement, loc = winner->getPathLocation(), hOffset, vOffset = winner->getPathVector();
	double theta;
	
	vOffset.Normalise();
	hOffset = vOffset.perp();
	theta = atan2(vOffset.y, vOffset.x);
	//loc = loc * (tilemap.getTileWidth() / kScale);
	short count = 0;
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
		if (count % 2 == 1) {
			placement = loc - hOffset * (0.55 * (*i)->car.getHeight() / kScale);
			loc -= vOffset * ((*i)->car.getWidth() / kScale);
		}
		else {
			placement = loc + hOffset * (0.55 * (*i)->car.getHeight() / kScale);
		}
        (*i)->car.setLocation(placement);
		(*i)->car.setOrientation(theta);
        (*i)->car.setVisible(true);
		(*i)->car.kill();
		count++;
    }
    
    playersInGame.clear();
	winner->updatePathLocation();
    for (list<Player*>::iterator i = allPlayersList.begin(); i != allPlayersList.end(); i++) {
        playersInGame.push_back(*i);
        (*i)->reset(startLineA, startLineB, winner);
    }
    
    Tuple middle(0.0, 0.0);
    
    for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++) {
        middle.x += (*i)->car.getLocation_wc().x * kScale;
        middle.y += (*i)->car.getLocation_wc().y * kScale;
    }
    middle.x /= playersInGame.size();
    middle.y /= playersInGame.size();
    
    views.back().view->scrollTo(floor(middle.x), floor(middle.y));
}
